#include "mbed.h"
#include "mt9d111.h"

#define uint32 unsigned long
#define uint16 unsigned short
#define byte char

Serial pc(USBTX, USBRX); // tx, rx

DigitalOut led1(LED1);
DigitalOut led2(LED2);
DigitalOut led3(LED3);
DigitalOut led4(LED4);
DigitalOut check(p20);
PwmOut fmclck(p21);
DigitalIn fvalid(p8);
DigitalIn dvalid(p30);
DigitalIn lvalid(p29);
DigitalOut storeImage(p28);

//InterruptIn frameValidIRQ(p8);
//InterruptIn lineValidIRQ(p30);
// DigitalIn pixelClock(p18);
// InterruptIn dataValidIRQ(p18);
// BusIn pixelData (p17, p16, p15, p14, p13, p12, p11, p8);
// LocalFileSystem local("local");               // Create the local filesystem under the name "local"

void printRegisters()
{
        printRegister(JPEG_FRONTEND_CONFIG_REGISTER,"JPEG_FRONTEND_CONFIG_REGISTER");
        printRegister(JPEG_CORE_CONFIG_REGISTER,"JPEG_CORE_CONFIG_REGISTER");               
        printRegister(JPEG_ENCODER_BYPASS,"JPEG_ENCODER_BYPASS"); 
        printRegister(OUTPUT_CONFIGURATION,"OUTPUT_CONFIGURATION"); 
        printRegister(PCLK1_PCLK2_CONFIGURATION,"PCLK1_PCLK2_CONFIGURATION"); 
        printRegister(PCLK3_CONFIGURATION,"JPEG_CORE_CONFIG_REGISTER");         
        printRegister(JPEG_CONTROL_REGISTER,"JPEG_CONTROL_REGISTER");
        printRegister(JPEG_STATUS_REGISTER_0,"JPEG_STATUS_REGISTER_0");
        printRegister(JPEG_STATUS_REGISTER_1,"JPEG_STATUS_REGISTER_1");
        printRegister(JPEG_STATUS_REGISTER_2,"JPEG_STATUS_REGISTER_2");        
        printRegister(CLOCK,"CLOCK");
        printRegister(CONTEXT_CONTROL,"CONTEXT_CONTROL");
        printRegister(OUTPUT_FORMAT,"OUTPUT_FORMAT");
}

void CaptureImage()
{        
        unsigned short state, value;        

        pc.printf("Prepare for capture\r\n");        
        writeRegister(ROW_SPEED,0x103,0x103);        
        writeRegister(CONTEXT_CONTROL,0xB);
        writeRegister(JPEG_CONTROL_REGISTER,0x1); // jpeg at next frame
        writeRegister(OUTPUT_CONFIGURATION,0x40);
        writeRegister(PCLK1_PCLK2_CONFIGURATION,0x80F);
        writeRegister(PCLK3_CONFIGURATION,0x4);   
       // writeMCUMemory(MODE_FIFO_CONFIG0_B,0x46);
        
        
        value = usReadRegister(JPEG_STATUS_REGISTER_0);
        if (value & 0x1)
        {
          writeRegister(JPEG_STATUS_REGISTER_0,0x11);
        }
        
        writeMCUMemory(MODE_CONTEXT,1);
        
        // configure jpeg
        writeMCUMemory(MODE_MODE_CONFIG,0);   
        writeRegister(JPEG_ENCODER_BYPASS,0);        
        
        writeMCUMemory(SEQ_CAPTUREPARAMS_NUMFRAMES,1);          
        writeMCUMemory(SEQ_CMD,5); // refresh
        
        // switch to capture mode
        writeMCUMemory(SEQ_CAPTUREPARAMS_MODE,0);                        
        
        pc.printf("Start capture\r\n");
        writeMCUMemory(SEQ_CMD,2); // do capture
                
        /* capture in progress when state > preview */        
        state = readMCU(SEQ_STATE);                         
        while (state >= 3)
        {           
           state = readMCU(SEQ_STATE);                   
        }           
       
        /* capture done. Wait until state is preview again */
        while (state != 3)
        {
          state = readMCU(SEQ_STATE);          
        }
        
        pc.printf("Capture done\r\n");        
      
}

void enablePWMClock()
{
   // pwm
    LPC_PWM1->TCR = (1 << 1);               // Reset counter, disable PWM
    LPC_SC->PCLKSEL0 &= ~(0x3 << 12);  
    LPC_SC->PCLKSEL0 |= (1 << 12);          // Set peripheral clock divider to /1, i.e. system clock
    LPC_PWM1->MR0 = 18;                     // Match Register 0 is shared period counter for all PWM1
    LPC_PWM1->MR6 = 9;                      // Pin 21 is PWM output 6, so Match Register 6
    LPC_PWM1->LER |= 1;                     // Start updating at next period start
    LPC_PWM1->TCR = (1 << 0) || (1 << 3);   // Enable counter and PWM    
}

int main() 
{ 
    int version=0, fw_version=0;
    int i = 0;
    char result[2];
    
    pc.printf("Starting MT9D111 test app\n\r");   
    enablePWMClock();

    pc.printf("Init..\n\r");
    mt9d11_init();
    
    readRegister(CHIP_VERSION, result);
    version = (result[0] << 8) + (result[1] & 0xFF);  
    
    if (version==CHIP_ID)
    {
        pc.printf("Invalid chip version: 0x%x. Expecting 0x%x\n\r",version,CHIP_ID);
        return 0;
    }
       
    readMCUMemory(MON_VER, result);     
    fw_version = (result[0] << 8) + (result[1] & 0xFF);             
    pc.printf("Fw: %i\n\r",fw_version);     
    pc.printf("Chip: %i\n\r",version);                   
      
     for (i=0;i<5000;i--)
     {                     
        CaptureImage();        
        wait(10);
    }            
}